import React, { useState, FC, PropsWithChildren, useEffect, useRef } from 'react';
import classNames from 'classnames';
import { HiDotsVertical } from 'react-icons/hi';
import {
  autoUpdate,
  flip,
  FloatingPortal,
  offset,
  shift,
  useClick,
  useDismiss,
  useFloating,
  useInteractions,
  // eslint-disable-next-line import/no-unresolved
} from '@floating-ui/react';
import MenuItemGroup from './MenuItemGroup';
import MenuItem from './MenuItem';

type Props = {
  id: string;
  isOpen?: boolean;
  popout?: JSX.Element;
  onClick?: (e?: React.MouseEvent<HTMLElement>) => void;
};

const RowDropdownMenu: FC<PropsWithChildren<Props>> = ({
                                                         id,
                                                         isOpen,
                                                         popout,
                                                         children,
                                                         onClick,
                                                       }: PropsWithChildren<Props>) => {
  const [showMenu, setShowMenu] = useState<boolean>(isOpen ?? false);
  const [showPopout, setShowPopout] = useState<boolean>(false);
  const ref = useRef<HTMLDivElement>(null);

  const { refs, floatingStyles, context } = useFloating({
    open: showMenu,
    onOpenChange: setShowMenu,
    placement: 'bottom-end',
    middleware: [offset(8), flip(), shift()],
    whileElementsMounted: autoUpdate,
  });

  const click = useClick(context);
  const dismiss = useDismiss(context);

  const { getReferenceProps, getFloatingProps } = useInteractions([click, dismiss]);

  useEffect(() => {
    if (isOpen != null) {
      setShowMenu(isOpen);
    }
  }, [isOpen]);

  const handleClick = (e: React.MouseEvent<HTMLElement>) => {
    e.stopPropagation();
    setShowPopout(true);
    setShowMenu(!showMenu);

    if (onClick) {
      onClick();
    }
  };

  const onHoverOrFocus = () => {
    setShowPopout(true);
  };

  const onUnhoverOrBlur = () => {
    setShowPopout(false);
  };

  const renderPopout = () => (
    <div
      data-test-id='row-dropdown-menu-popout'
      className={classNames('absolute opacity-0 transition-all ease-in-out delay-200 duration-500', {
        'opacity-100 -translate-x-8 z-10': showPopout || showMenu,
      })}
    >
      {popout}
    </div>
  );

  const renderTriggerButton = () => (
    <button
      ref={refs.setReference}
      data-test-id={id}
      type='button'
      className={classNames(
        'absolute rounded-sm p-1 cursor-pointer text-primary-500 transition ease-in-out duration-500 hover:bg-primary-50 hover:-rotate-90',
        {
          '-rotate-90 z-10': showPopout || showMenu,
        }
      )}
      onClick={handleClick}
      {...getReferenceProps()}
    >
      <HiDotsVertical size='1.25em' />
    </button>
  );

  return (
    <div
      ref={ref}
      className='relative'
      aria-expanded={showMenu}
      onMouseOver={onHoverOrFocus}
      onFocus={onHoverOrFocus}
      onMouseOut={onUnhoverOrBlur}
      onBlur={onUnhoverOrBlur}
    >
      <div className='block relative'>
        {popout && renderPopout()}
        {renderTriggerButton()}
      </div>

      {showMenu && (
        <FloatingPortal>
          <div
            ref={refs.setFloating}
            data-test-id='row-dropdown-menu-options'
            className={classNames(
              'absolute top-8 -right-4 z-10 w-fit text-xs list-none bg-white rounded-lg border border-gray-200 shadow-sm'
            )}
            style={floatingStyles}
            {...getFloatingProps()}
          >
            <ul className='text-left' role='menu'>
              {children}
            </ul>
          </div>
        </FloatingPortal>
      )}
    </div>
  );
};

export { MenuItemGroup, MenuItem };
export default RowDropdownMenu;
